/*
 * Created by SharpDevelop.
 * User: 
 * Date: 24.10.2007
 * Time: 20:57
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace ColorButtonApp
{
	/// <summary>
	/// Description of ColorButton.
	/// </summary>
	public partial class ColorButton : Control
	{
		public ColorButton()
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			
			//
			// TODO: Add constructor code after the InitializeComponent() call.
			//
		}
		private Color m_color1 = Color.White; // 
		private Color m_color2 = Color.Blue;  // 
		
		public Color Color1
    	{
        	get { return m_color1; }
        	set { m_color1 = value; Invalidate(); }
    	}

    	public Color Color2
    	{
        	get { return m_color2; }
        	set { m_color2 = value; Invalidate(); }
    	}
    	
    	public override string Text
		{
			get	{ return base.Text; }
			set	{ base.Text = value; Invalidate(); }
		}
		
		private LinearGradientMode mode;
		private LinearGradientBrush brush;		
		
		private enum State
		{
			Normal,
			MouseOver,
			MouseClicked
		}
		private State CurrentState = State.Normal;
		
		protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
		{
			mode = LinearGradientMode.Vertical;
			
			//   
			SizeF textSize = e.Graphics.MeasureString(Text, base.Font);
			int textX = (int)(base.Size.Width / 2) - (int)(textSize.Width / 2);
			int textY = (int)(base.Size.Height / 2) - (int)(textSize.Height / 2);
			
			switch (CurrentState)
			{
				case State.Normal:
					PaintNormalState(e.Graphics);	
					break;
				case State.MouseOver:
					PaintHoverState(e.Graphics);
					break;
				case State.MouseClicked:
					PaintClickedState(e.Graphics);
					break;
			}
			
			e.Graphics.DrawString(Text, base.Font, new SolidBrush(base.ForeColor), textX + 1,textY + 1);
		}
		
		protected void PaintNormalState(Graphics g)
		{
				
			Rectangle newRect = new Rectangle(ClientRectangle.X + 1,
			                                  ClientRectangle.Y + 1,
			                                  ClientRectangle.Width - 2,
			                                  ClientRectangle.Height - 2);
			brush = new LinearGradientBrush(newRect, m_color1, m_color2, mode);
		
			g.FillRectangle(brush, newRect);
			g.DrawRectangle(new Pen(m_color2, 1), newRect);
				
		}
		
		protected void PaintHoverState(Graphics g)
		{
			Rectangle newRect = new Rectangle(ClientRectangle.X + 1,
			                                  ClientRectangle.Y + 1,
			                                  ClientRectangle.Width - 2,
			                                  ClientRectangle.Height - 2);
			brush = new LinearGradientBrush(newRect, m_color2, m_color1, mode);
		
			g.FillRectangle(brush, newRect);
			g.DrawRectangle(new Pen(m_color2, 1), newRect);

		}
		
		protected void PaintClickedState(Graphics g)
		{
			Rectangle newRect = new Rectangle(ClientRectangle.X + 1,
			                                  ClientRectangle.Y + 1,
			                                  ClientRectangle.Width - 2,
			                                  ClientRectangle.Height - 2);
			brush = new LinearGradientBrush(newRect, m_color2, m_color2, mode);
		
			g.FillRectangle(brush, newRect);
			g.DrawRectangle(new Pen(m_color2, 1), newRect);

		}	
		
		protected override void OnMouseLeave(System.EventArgs e)
		{
			CurrentState = State.Normal;
			Invalidate();
			base.OnMouseLeave(e);
		}
		protected override void OnMouseEnter(System.EventArgs e)
		{
			CurrentState = State.MouseOver;
			Invalidate();
			base.OnMouseEnter(e);
		}
		protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs e)
		{
			CurrentState = State.MouseClicked;
			Invalidate();
			base.OnMouseDown(e);
		}
		protected override void OnMouseUp(System.Windows.Forms.MouseEventArgs e)
		{
			CurrentState = State.MouseOver;
			Invalidate();
			base.OnMouseEnter(e);
		}
	}
}
